require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

require 'csv'

module StudioGame
  class Game
    def initialize(title)
      @title = title.capitalize
      @players = []
    end

    def high_score_entry(player)
      "#{player.name.ljust(20, '.')} #{player.score}"
    end


    def load_players(file)
      CSV.foreach(file) do |row|
        add_player(Player.new(row[0], row[1].to_i))
      end
    end

    def save_high_scores(output_file="high_scores.txt")
      File.open(output_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each do |player|
          file.puts high_score_entry(player)
        end
      end
    end

    def add_player(player_name)
      @players << player_name
    end

    def print_score
      puts "\n#{@title} high scores:"
      @players.sort.each do |player|
        puts high_score_entry(player)
      end
    end

    def strong_stats(strong_players)
      puts "\n#{strong_players.size} strong players:"
      strong_players.each do |player|
        puts "#{player.name} (#{player.health})"
      end
    end

    def wimpy_stats(wimpy_players)
      puts "\n#{wimpy_players.size} wimpy players:"
      wimpy_players.each do |player|
        puts "#{player.name} (#{player.health})"
      end
    end

    def total_treasure_points
      @players.reduce(0) do |sum, player|
        sum + player.points
      end
    end

    def print_stats
      puts "\n#{@title}'s totals:"
      puts "#{total_treasure_points} total points from treasures found."
      @players.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points."
        end
        puts "#{player.points} GRAND TOTAL POINTS."
      end
      puts "\n#{@title} statistics:"
      strong_players, wimpy_players = @players.partition { |player| player.strong? }
      strong_stats(strong_players)
      wimpy_stats(wimpy_players)
    end

    def play(rounds)
      TreasureTrove.list
      puts "\nThere are #{@players.size} players in #{@title}."
      @players.each do |p|
        puts p
      end
      1.upto(rounds) do |count|
        puts "\nRound #{count}"
        @players.each do |p|
          GameTurn.take_turn(p)
          puts p
        end
      end
    end
  end

  if __FILE__ == $PROGRAM_NAME
    game1 = Game.new("Chipmonks")

    player1 = Player.new("Marcus", 100)
    player2 = Player.new("Amanda", 100)
    game1.add_player(player1)
    game1.add_player(player2)
    game1.play(10)
    game1.print_stats
  end
end
