require_relative 'treasure_trove'
require_relative 'playable'

module StudioGame
  class Player

    include Playable

    attr_accessor :name, :health

    def initialize(name, health=100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new(0)
    end

    def to_s
      "I'm #{@name}, health = #{@health}, points = #{points}, and score = #{score}."
    end

    def points
      @found_treasures.values.reduce(0, :+)
    end

    def each_found_treasure
      @found_treasures.each do |name, points|
        yield Treasure.new(name, points)
      end
    end

    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
      puts "#{@name}'s treasures: #{@found_treasures}"
    end

    def <=>(other_player)
      other_player.score <=> score
    end

    def score
      @health + points
    end
  end

  if __FILE__ == $0
    player1 = Player.new("moe", 90)
    puts player1
    player1.w00t
    player1.w00t
    puts "#{player1.name}'s health is: #{player1.health}."
    player1.blam
    puts "#{player1.name}'s health is: #{player1.health}."
  end
end
